/*
 * LinShare is an open source filesharing software, part of the LinPKI software
 * suite, developed by Linagora.
 * 
 * Copyright (C) 2013 LINAGORA
 * 
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version, provided you comply with the Additional Terms applicable for
 * LinShare software by Linagora pursuant to Section 7 of the GNU Affero General
 * Public License, subsections (b), (c), and (e), pursuant to which you must
 * notably (i) retain the display of the “LinShare™” trademark/logo at the top
 * of the interface window, the display of the “You are using the Open Source
 * and free version of LinShare™, powered by Linagora © 2009–2013. Contribute to
 * Linshare R&D by subscribing to an Enterprise offer!” infobox and in the
 * e-mails sent with the Program, (ii) retain all hypertext links between
 * LinShare and linshare.org, between linagora.com and Linagora, and (iii)
 * refrain from infringing Linagora intellectual property rights over its
 * trademarks and commercial brands. Other Additional Terms apply, see
 * <http://www.linagora.com/licenses/> for more details.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License and
 * its applicable Additional Terms for LinShare along with this program. If not,
 * see <http://www.gnu.org/licenses/> for the GNU Affero General Public License
 * version 3 and <http://www.linagora.com/licenses/> for the Additional Terms
 * applicable to LinShare software.
 */
package org.linagora.linshare.repository.hibernate;

import java.util.Date;

import org.junit.Assert;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.linagora.linshare.core.domain.constants.LinShareTestConstants;
import org.linagora.linshare.core.domain.entities.AbstractDomain;
import org.linagora.linshare.core.domain.entities.Account;
import org.linagora.linshare.core.domain.entities.Guest;
import org.linagora.linshare.core.domain.entities.Tag;
import org.linagora.linshare.core.exception.BusinessException;
import org.linagora.linshare.core.repository.AbstractDomainRepository;
import org.linagora.linshare.core.repository.AccountRepository;
import org.linagora.linshare.core.repository.TagRepository;
import org.linagora.linshare.service.LoadingServiceTestDatas;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.AbstractTransactionalJUnit4SpringContextTests;




@ContextConfiguration(locations={"classpath:springContext-test.xml", 
		"classpath:springContext-datasource.xml",
		"classpath:springContext-repository.xml"})
public class TagRepositoryImplTest extends AbstractTransactionalJUnit4SpringContextTests {

	
	private static final Logger logger = LoggerFactory.getLogger(TagRepositoryImplTest.class);
	
	
	@Autowired
	private AbstractDomainRepository abstractDomainRepository;
	
	@Autowired
	private AccountRepository<Account> accountRepository;
	
	@Autowired
	private TagRepository tagRepository;
	
    private static final String FIRST_NAME = "first name";
    private static final String LAST_NAME = "last name";
    private static final String MAIL = "mail";
    private static final String PASSWORD = "password";
    
	// Members
    private AbstractDomain rootDomain;
    private Account account;
	
	
	@Before
	public void setUp() throws Exception {
		logger.debug("Begin setUp");
		rootDomain = abstractDomainRepository.findById(LoadingServiceTestDatas.sqlRootDomain);
		account = new Guest(FIRST_NAME, LAST_NAME, MAIL, PASSWORD, true, "comment");
		account.setDomain(rootDomain);
		account.setCreationDate(new Date());
		account.setLocale(rootDomain.getDefaultLocale());
		accountRepository.create(account);
		
		logger.debug("End setUp");
	}

	@After
	public void tearDown() throws Exception {
		logger.debug("Begin tearDown");
		
		accountRepository.delete(account);
		
		logger.debug("End tearDown");
	}
	
	
	@Test
	public void testCreateTag() throws BusinessException{
		logger.info(LinShareTestConstants.BEGIN_TEST);
		Tag tag = new Tag(account, "toto");
		
		tagRepository.create(tag);
		tagRepository.delete(tag);
		
		logger.debug(LinShareTestConstants.END_TEST);
	}
	
	
	@Test
	public void testFindTag() throws BusinessException{
		logger.info(LinShareTestConstants.BEGIN_TEST);
		Tag tag = new Tag(account, "toto");
		
		tagRepository.create(tag);
		
		Assert.assertNull(tagRepository.findByOwnerAndName(account, "tota"));
		Assert.assertNotNull(tagRepository.findByOwnerAndName(account, "toto"));
		
		tagRepository.delete(tag);
		logger.debug(LinShareTestConstants.END_TEST);
	}
	
	
}
